/*
**		Automatic Directory Cleaner
**
**		(c) 1995 Martin Mares <mj@k332.feld.cvut.cz>
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <dirent.h>
#include <time.h>
#include <sys/stat.h>
#include <unistd.h>

static char path[PATH_MAX];
static time_t now;

static void
log(char *text, char *context)
{
	printf("%s %s\n", text, context);
}

static void
logerr(char *text, char *context)
{
	printf("%s %s: %s\n", text, context, strerror(errno));
}

static int
go(char *point, int remove, int tim)
{
DIR *d;
struct dirent *e;
int leaving = 0;
	*point = '.';
	point[1] = 0;
	d = opendir(path);
	if (d)
		{
		while (e = readdir(d))
			{
			char *pt = point + strlen(e->d_name);
				if (pt >= path + PATH_MAX - 2)
					{
					point[0] = 0;
					log("Path too long while scanning", path);
					leaving = 1;
					}
				else
					{
					struct stat s;
					int flag;
						strcpy(point, e->d_name);
						if (stat(path, &s))
							{
							log("Unable to stat", path);
							leaving = 1;
							}
						else
							{
							if (!strcmp(e->d_name, ".") || !strcmp(e->d_name, ".."))
								flag = 2;
							else if (S_ISDIR(s.st_mode))
								{
								pt[0] = '/';
								flag = go(pt+1, 1, tim);
								}
							else
								flag = 0;
							if (now - tim > s.st_mtime && !flag)
								{
								log("Deleting", path);
								if (unlink(path))
									{
									logerr("Error deleting", path);
									leaving = 1;
									}
								}
							else if (flag != 2)
								leaving = 1;
							}
					}
			}
		closedir(d);
		}
	else
		log("Unable to read directory", path);
	if (!remove)
		leaving = 1;
	if (!leaving)
		{
		point[0] = 0;
		point[-1] = 0;
		log("Deleting directory", path);
		chmod(path, 777);
		if (rmdir(path))
			{
			logerr("Error deleting directory", path);
			leaving = 1;
			}
		point[-1] = '/';
		}
	return leaving;
}

static void
do_clean(char *arg)
{
char *name = arg;
char *timo = strchr(name, '=');
int timeout;
	if (!timo || !timo[1])
		{
		fprintf(stderr, "Missing timeout value in `%s' !\n", arg);
		exit(1);
		}
	*timo++ = 0;
	timeout = atol(timo);
	if (!timeout)
		{
		fprintf(stderr, "Invalid timeout: `%s' !\n", timo);
		exit(1);
		}
	strcpy(path, arg);
	name = path + strlen(path);
	if (name == path || name[-1] != '/')
		*name++ = '/';
	go(name, 0, timeout);
}

void
main(int argc, char **argv)
{
int i;
char *chr = NULL;
	time(&now);
	if (argc > 1 && *argv[1] == '@' && argv[1][1])
		{
		chr = argv[1]+1;
		argv++;
		argc--;
		}
	if (argc < 2 || !strcmp(argv[1], "--help"))
		{
		printf("Automatic Directory Cleaner 1.0 (c) 1995 Martin Mares <mj@k332\
.feld.cvut.cz>\n\n\
Usage: autoclean [@<chroot-dir>] <dir>=<timeout-seconds> ...\n");
		exit(1);
		}
	printf("AutoClean started on %s", ctime(&now));
	if (chr)
		{
		printf("Chrooting to %s\n", chr);
		if (chroot(chr))
			{
			perror("chroot() failed");
			exit(2);
			}
		if (chdir("/"))
			{
			perror("chdir() failed");
			exit(2);
			}
		}
	for(i=1; i<argc; i++)
		do_clean(argv[i]);
	puts("Done.");
}
